import ropchain
rax = ropchain.Regs.rax
rbx = ropchain.Regs.rbx
rcx = ropchain.Regs.rcx
rdx = ropchain.Regs.rdx
rdi = ropchain.Regs.rdi
rsi = ropchain.Regs.rsi
rbp = ropchain.Regs.rbp
rsp = ropchain.Regs.rsp
r8 = ropchain.Regs.r8
r9 = ropchain.Regs.r9
r10 = ropchain.Regs.r10
r11 = ropchain.Regs.r11
r12 = ropchain.Regs.r12
r13 = ropchain.Regs.r13
r14 = ropchain.Regs.r14
r15 = ropchain.Regs.r15
eax = ropchain.Regs.eax
ebx = ropchain.Regs.ebx
ecx = ropchain.Regs.ecx
edx = ropchain.Regs.edx
edi = ropchain.Regs.edi
esi = ropchain.Regs.esi
ebp = ropchain.Regs.ebp
esp = ropchain.Regs.esp

regDict = {
        'rax': rax,
        'rbx': rbx,
        'rcx': rcx,
        'rdx': rdx,
        'rdi': rdi,
        'rsi': rsi,
        'rbp': rbp,
        'rsp': rsp,
        'r8': r8,
        'r9': r9,
        'r10': r10,
        'r11': r11,
        'r12': r12,
        'r13': r13,
        'r14': r14,
        'r15': r15,
        'eax': eax,
        'ebx': ebx,
        'ecx': ecx,
        'edx': edx,
        'edi': edi,
        'esi': esi,
        'ebp': ebp,
        'esp': esp,
        }
